from . import backend
from .hd_fs import curve_within_radii

import numpy as np
import itertools
import progressbar

def frechet_median(T, epsilon):
    coreset = backend.onemedian_coreset(T, epsilon/2.)
    cost = coreset.cost
    lower = cost/6
    lambd = coreset.lambd
    Lambd = coreset.Lambd
    curves = list(coreset.curves())
    curves_dist = list(set(curves))
    coreset_dist = backend.Curves()
    coreset_dist_multiplicities = [curves.count(elem) for elem in curves_dist]
    for curve_ind in curves_dist:
        coreset_dist.add(T[int(curve_ind)])
    radiis = list()
    length = 1
    for i in range(0, len(curves_dist)):
        range_ = np.linspace(0, cost, num=np.ceil(cost/(epsilon*cost/24*lambd[curves_dist[i]]/Lambd)), endpoint=True)
        length *= len(range_)
        radiis.append(range_)
    candidates = list()
    count = 0
    with progressbar.ProgressBar(max_value=length) as bar:
        for radii in itertools.product(*radiis):
            count += 1
            bar.update(count)
            rsum = np.sum(radii)
            if rsum < lower or rsum > cost:
                continue
            candidate = curve_within_radii(coreset_dist, radii)
            if candidate[0]:
                candidates.append(candidate[1])
    min_cost_curve = 0
    min_cost = np.infty
    for i in progressbar.progressbar(range(0, len(candidates))):
        cost = 0
        for j in range(0, len(coreset_dist)):
            cost += coreset_dist_multiplicities[j] * Lambd/lambd[curves_dist[j]] * 1/len(curves) * backend.continuous_frechet(candidates[i], coreset_dist[j]).value
        if cost < min_cost:
            min_cost_curve = i
            min_cost = cost
    return candidates[min_cost_curve]



import os
import re
import sys
import sysconfig
import platform
import subprocess

from glob import glob
from distutils.version import LooseVersion
from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
import setuptools

this_dir = os.path.dirname(os.path.abspath(__file__))

class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=this_dir):
        Extension.__init__(self, name, 
                           include_dirs=[os.path.join(this_dir, 'include'), os.path.join(this_dir, 'pybind11')],
                           sources=sorted(glob("src/*.cpp")))
        self.sourcedir = os.path.abspath(sourcedir)

class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '.', '--version'])
        except OSError:
            raise RuntimeError(
                "CMake must be installed to build the following extensions: " +
                ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r'version\s*([\d.]+)',
                                         out.decode()).group(1))
            if cmake_version < '3.1.0':
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(
            os.path.dirname(self.get_ext_fullpath(ext.name)))
            
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(
                cfg.upper(),
                extdir)]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get('CXXFLAGS', ''),
            self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.call(['git', 'init'])
        subprocess.call(['git', 'submodule', 'add', 'https://github.com/pybind/pybind11.git'])
        subprocess.call(['git', 'submodule', 'update', '--init', '--recursive'])
        subprocess.check_call(['cmake', "{}".format(ext.sourcedir)] + cmake_args,
                              cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args,
                              cwd=self.build_temp)

setup(
    name='Fred-Frechet',
    version='1.7.5',
    author='Dennis Rohde',
    author_email='dennis.rohde@tu-dortmund.de',
    description='A fast, scalable and light-weight C++ Fréchet distance library, exposed to python and focused on (k,l)-clustering of polygonal curves.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="http://fred.dennisrohde.work",
    packages=setuptools.find_packages(),
    ext_package="Fred",
    ext_modules=[CMakeExtension('backend')],
    install_requires=['cvxopt', 'matplotlib'],
    cmdclass=dict(build_ext=CMakeBuild),
    data_files=[('pybind11', ['.gitmodules'])],
    zip_safe=False,
)
